# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Author: Nikola Rokvic
# Created: 18.10.2021
# Copyright 2021 InferStat Ltd

import http.client
import json
import pandas as pd
import requests
import markdown
import pathlib

"""Scripts made to allow package users to use the InferTrade API easier"""


def remove_at(i, s):
    """Removes character from string at provided position"""
    return s[:i] + s[i + 1 :]


def parse_csv_file(file_name: str = None, file_location: str = None):
    """Function reads provided CSV file (found in package folder) and returns data parsed to dict"""
    if file_name is None and file_location is None:
        raise ValueError("Please provide file name or file location")
    if file_name is not None:
        if ".csv" not in str(file_name):
            raise ValueError("Please provide CSV file or add .csv to file name")
        file_path = str(pathlib.Path(__file__).parent.parent.parent.resolve()) + "/" + file_name
    elif file_location is not None:
        if ".csv" not in str(file_location):
            raise ValueError("Please provide CSV file or add .csv to file location")
        file_path = file_location
    dataframe = pd.read_csv(file_path)
    dictionary = dataframe.to_dict('list')
    return dictionary


def find_request_method(html, request_name: str):
    """Function finds request method in provided HTML file"""
    a = ""
    for _ in html:
        if _ is not "\n":
            a = a + str(_)
        elif ("POST " + request_name) in a:
            return "POST", html.index(("POST " + request_name))
        elif ("GET " + request_name) in a:
            return "GET", html.index(("GET " + request_name))
        else:
            a = ""

    raise ValueError("Could not find request method in documentation")


def scrape_request_body(request_name: str):
    """Converts API_GUIDANCE.md file into HTML"""
    md_location = str(pathlib.Path(__file__).parent.parent.parent.resolve()) + "/API_GUIDANCE.md"
    with open(md_location, "r") as f:
        text = f.read()
        html = markdown.markdown(text)

    request_method, request_index = find_request_method(html=html, request_name=request_name)

    request_body = retrieve_request_body(request_index=request_index, html=html)
    return request_body, request_method


def retrieve_request_body(request_index: int, html):
    """Returns default request body from API_GUIDANCE.md file"""
    request_body = ""
    a = ""
    body = False
    for _ in range(request_index, len(html)):
        if html[_] != " ":
            if body is False:
                a = a + html[_]
        elif body is False:
            a = ""
        if a == "json.dumps({":
            if (str(html[_] + html[_ + 1])) == "})":
                return request_body
            body = True
            request_body = request_body + html[_]
    return ""


def check_api_status():
    """Function checks current status of InferTrade API"""
    conn = http.client.HTTPSConnection("prod.api.infertrade.com")
    payload = ""
    headers = {"Content-Type": "application/json"}
    conn.request("GET", "/status", payload, headers)
    res = conn.getresponse()
    data = res.read()
    status = data.decode("utf-8")
    return status


def check_float(string_rep):
    """Function checks if provided string represents float number"""
    try:
        float(string_rep)
        return True
    except ValueError:
        return False


def find_and_replace_bracket(test_str):
    test_str = test_str
    open = 0
    open_positions = dict()
    ordered_keys = list()
    for _, i in enumerate(test_str):
        if i == "[":
            open += 1
        elif i == "]":
            open -= 1
            if open == 0:
                b = test_str.find("[")
                open_positions[b] = _
                ordered_keys.append(b)
                test_str = test_str[:b] + "]" + test_str[b + 1 :]

    for i in range(0, (len(ordered_keys))):
        test_str = (
            test_str[: ordered_keys[(len(ordered_keys) - 1) - i]]
            + "placeholder"
            + test_str[(open_positions[ordered_keys[(len(ordered_keys) - 1) - i]] + 1) :]
        )
    return test_str


def string_to_dict(string, in_recursion: int = 0, fillers: dict() = None):
    """Function turns scraped request body into a dictionary"""
    dictionary = dict()
    value = None
    key = None
    list_key = list()
    list_value = list()
    position = None
    str_len = len(string)
    i = 0
    for i, _ in enumerate(string):
        if position is not None:
            if i != str_len - len(string):
                continue
            else:
                position = None
        if _ == ":":
            if string.find(_) != 0:
                key = string[: (string.find(_))]
            string = string[(string.find(_) + 1) :]
        elif _ == ",":
            if string.find(_) != 0:
                value = string[: string.find(_)]
            string = string[(string.find(_) + 1) :]
        elif _ == "}":
            if string.find(_) != 0:
                value = string[: string.find(_)]
            string = string[(string.find(_) + 1) :]
            if value is not None and key is not None:
                list_value.append(value)
                list_key.append(key)
                key = str(list_key.pop())
                a = list_value.pop()
                if isinstance(a, dict):
                    dictionary[key] = a
                elif check_float(a):
                    a = "placeholder"
                    dictionary[key] = a
                else:
                    dictionary[key] = a
                if fillers is not None and key in fillers.keys():
                    dictionary[key] = fillers[key]
            return dictionary, string, i
        elif _ == "{":
            string = string[(string.find(_) + 1) :]
            in_recursion += 1
            value, string, position = string_to_dict(string, in_recursion=in_recursion, fillers=fillers)
            in_recursion -= 1
            if len(string) > 1:
                if string[0] == "}":
                    string = string[1:]
                    if value is not None and key is not None:
                        list_value.append(value)
                        list_key.append(key)
                        key = str(list_key.pop())
                        a = list_value.pop()
                        if isinstance(a, dict):
                            dictionary[key] = a
                        elif check_float(a):
                            a = "placeholder"
                            dictionary[key] = a
                        else:
                            dictionary[key] = a
                        if fillers is not None and key in fillers.keys():
                            dictionary[key] = fillers[key]
                    return dictionary, string, i

        if value is not None and key is not None:
            list_value.append(value)
            list_key.append(key)
            key = str(list_key.pop())
            a = list_value.pop()
            if isinstance(a, dict):
                dictionary[key] = a
            elif check_float(a):
                a = "placeholder"
                dictionary[key] = a
            else:
                dictionary[key] = a
            if fillers is not None and key in fillers.keys():
                dictionary[key] = fillers[key]
            key = None
            value = None

    return dictionary, string, i


def convert_string(string: str, fillers: dict() = None):
    """Makes retrieved string compatible with "string_to_dict" method and passes converted string to "string_to_dict" """
    body = remove_at(0, string)
    new_body = "".join(body.splitlines())
    new_body = new_body.replace(" ", "")
    new_body = new_body.replace('"', "")
    new_body = find_and_replace_bracket(new_body)
    new_body = "".join(new_body.splitlines())
    new_body, string, pos = string_to_dict(new_body, fillers=fillers)
    return dict(new_body)


def execute_it_api_request(
    request_name: str,
    api_key: str,
    request_body: dict() = None,
    header: dict() = None,
    additional_data: list() = None,
    Content_Type: str = "application/json",
    selected_module: str = "requests",
    execute_request: bool = True
):
    """Combines data and execute InferTrade API request, returns response"""

    status = check_api_status()
    if "running" not in status:
        raise Exception("Failed to establish connection to InferTrade API")

    scraped_body, method = scrape_request_body(request_name)
    if request_body is not None:
        new_body = request_body
    elif scraped_body != "":
        new_body = convert_string(scraped_body, fillers=additional_data)
    else:
        new_body = scraped_body

    payload = json.dumps(new_body)

    if execute_request is False:
        return payload

    if header is None:
        headers = {"Content-Type": Content_Type, "x-api-key": api_key}
    else:
        headers = header

    if selected_module == "http.client":
        conn = http.client.HTTPSConnection("prod.api.infertrade.com")
        conn.request(method, "/", payload, headers)
        res = conn.getresponse()
        data = res.read()
        response = data.decode("utf-8")
    elif selected_module == "requests":
        url = "https://prod.api.infertrade.com/"
        response = requests.request(method, url, headers=headers, data=payload)

    return response



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Author: Nikola Rokvic
# Created: 17.08.2021
# Copyright 2021 InferStat Ltd

from copy import deepcopy

import numpy as np
import pandas as pd
from sklearn.pipeline import make_pipeline
from infertrade.algos.community import scikit_signal_factory
from infertrade.algos import algorithm_functions, ta_adaptor
from infertrade.utilities.operations import PricePredictionFromSignalRegression, PositionsFromPricePrediction
from infertrade.utilities.performance import calculate_portfolio_performance_python
from infertrade.algos.external.ta_regressions import ta_export_regression_allocations
from infertrade.api import Api


def calculate_ta_regression_allocation(dataframe: pd.DataFrame, rule_name: str):
    """Used to implement ta regression allocations with market data."""
    df = deepcopy(dataframe)
    percent_gain = [0]

    ta_alloc = ta_export_regression_allocations[rule_name]["function"]
    df_with_allocations = ta_alloc(df)
    df_with_performance = calculate_portfolio_performance_python(df_with_allocations, detailed_output=True)
    for i in range(1, len(df_with_performance["portfolio_return"])):
        percent_gain.append(
            (df_with_performance["portfolio_return"][i] - df_with_performance["portfolio_return"][i - 1]) * 100
        )
    df_with_performance = df_with_performance.assign(percent_gain=percent_gain)
    return df_with_performance


def calculate_infertrade_allocation(dataframe: pd.DataFrame, rule_name: str):
    """Used to implement infertrade allocations with market data."""
    df = deepcopy(dataframe)
    percent_gain = [0]

    df_with_allocations = Api.calculate_allocations(df=df, name_of_strategy=rule_name, name_of_price_series="close")
    df_with_performance = calculate_portfolio_performance_python(df_with_allocations, detailed_output=True)
    for i in range(1, len(df_with_performance["portfolio_return"])):
        percent_gain.append(
            (df_with_performance["portfolio_return"][i] - df_with_performance["portfolio_return"][i - 1]) * 100
        )
    df_with_performance = df_with_performance.assign(percent_gain=percent_gain)
    return df_with_performance


def calculate_infertrade_signal(dataframe: pd.DataFrame, rule_name: str):
    """Used to implement infertrade signals with market data."""
    df = deepcopy(dataframe)
    percent_gain = [0]

    pipeline = make_pipeline(
        scikit_signal_factory(algorithm_functions["infertrade"]["signal"][rule_name]["function"]),
        PricePredictionFromSignalRegression(),
        PositionsFromPricePrediction(),
    )

    df = pipeline.fit_transform(df)
    df_with_performance = calculate_portfolio_performance_python(df, detailed_output=True)
    for i in range(1, len(df_with_performance["portfolio_return"])):
        percent_gain.append(
            (df_with_performance["portfolio_return"][i] - df_with_performance["portfolio_return"][i - 1]) * 100
        )
    df_with_performance = df_with_performance.assign(percent_gain=percent_gain)
    return df_with_performance


def calculate_ta_allocation(dataframe: pd.DataFrame, rule_name: str):
    "Used to implement ta allocations with market data"
    df = deepcopy(dataframe)
    percent_gain = [0]

    df_with_allocations = Api.calculate_allocations(df=df, name_of_strategy=rule_name, name_of_price_series="close")
    df_with_performance = calculate_portfolio_performance_python(df_with_allocations, detailed_output=True)
    for i in range(1, len(df_with_performance["portfolio_return"])):
        percent_gain.append(
            (df_with_performance["portfolio_return"][i] - df_with_performance["portfolio_return"][i - 1]) * 100
        )
    df_with_performance = df_with_performance.assign(percent_gain=percent_gain)
    return df_with_performance


def calculate_ta_signal(dataframe: pd.DataFrame, rule_name: str):
    "Used to implement ta signals with market data"
    df = deepcopy(dataframe)
    percent_gain = [0]

    adapted_rule = ta_adaptor(algorithm_functions["ta"]["signal"][rule_name]["class"], rule_name)

    pipeline = make_pipeline(
        scikit_signal_factory(adapted_rule), PricePredictionFromSignalRegression(), PositionsFromPricePrediction(),
    )
    df = pipeline.fit_transform(df)
    df_with_performance = calculate_portfolio_performance_python(df, detailed_output=True)
    for i in range(1, len(df_with_performance["portfolio_return"])):
        percent_gain.append(
            (df_with_performance["portfolio_return"][i] - df_with_performance["portfolio_return"][i - 1]) * 100
        )
    df_with_performance = df_with_performance.assign(percent_gain=percent_gain)
    return df_with_performance


def export_performance_df(
    dataframe: pd.DataFrame, rule_name: str = None, second_df: pd.DataFrame = None, relationship: str = None
) -> pd.DataFrame:
    """
    Function used to calculate portfolio performance for data after calculating a trading signal/rule and relationship.
    """
    if rule_name is not None:
        if rule_name in algorithm_functions["infertrade"]["allocation"].keys():
            used_calculation = calculate_infertrade_allocation

        elif rule_name in algorithm_functions["ta"]["signal"].keys():
            used_calculation = calculate_ta_signal

        elif rule_name in algorithm_functions["ta"]["allocation"].keys():
            used_calculation = calculate_ta_allocation

        elif rule_name in algorithm_functions["infertrade"]["signal"].keys():
            used_calculation = calculate_infertrade_signal

        elif rule_name in ta_export_regression_allocations.keys():
            used_calculation = calculate_ta_regression_allocation

        elif rule_name not in algorithm_functions:
            raise ValueError("Algorithm not found")

        df_with_performance = used_calculation(dataframe=dataframe, rule_name=rule_name)
    else:
        df_with_performance = dataframe
    if relationship is not None:
        if second_df is not None:
            if rule_name is not None:
                second_df_with_performance = used_calculation(dataframe=second_df, rule_name=rule_name)
            else:
                second_df_with_performance = second_df

            second_df_with_relationship = calculate_infertrade_allocation(
                dataframe=second_df_with_performance, rule_name=relationship
            )

            df_with_relationship = calculate_infertrade_allocation(
                dataframe=df_with_performance, rule_name=relationship
            )

            complete_relationship = df_with_relationship.append(second_df_with_relationship, ignore_index=False)
            return complete_relationship
        else:
            df_with_relationship = calculate_infertrade_allocation(
                dataframe=df_with_performance, rule_name=relationship
            )
            return df_with_relationship
    else:
        return df_with_performance


def sort_dict_desc(dict_of_dfs: dict, column_name: str) -> dict:
    """Function used to sort dictionary containing pandas dataframes in descending order based on supplied column name"""
    sum_of_requested_columns = dict()
    for _ in dict_of_dfs.keys():
        sum_of_requested_columns[_] = dict_of_dfs[_][column_name].sum()

    sorted_dict_of_dfs = dict(sorted(sum_of_requested_columns.items(), key=lambda item: item[1], reverse=True))

    return sorted_dict_of_dfs


def evaluate_cross_prediction(
    list_of_dfs_of_asset_prices: list,
    column_to_sort: str = "percent_gain",
    number_of_results: int = 0,
    export_as_csv: bool = True,
) -> list:
    """A function to evaluate any predictive relationships between the supplied asset time series, with rankings exported to CSV.
    Supplied time series are evaluated and sorted in descending order with return percentage being the compared value"""

    relationship_names = []
    dict_of_relationship_performance = dict()
    for ii_package in algorithm_functions:
        for ii_algo_type in algorithm_functions[ii_package]:
            for rule in algorithm_functions[ii_package][ii_algo_type]:
                if "relationship" in rule:
                    relationship_names.append(rule)

    if len(list_of_dfs_of_asset_prices) < 2:
        raise ValueError("At least 2 time series needed to evaluate predictive relationships")
    else:
        for relationship in relationship_names:
            for first_asset_df_index in range(0, len(list_of_dfs_of_asset_prices)):
                for second_asset_df_index in range(0, len(list_of_dfs_of_asset_prices)):
                    if first_asset_df_index == second_asset_df_index:
                        continue
                    name = str(first_asset_df_index) + "_" + str(second_asset_df_index) + "_" + str(relationship)
                    dict_of_relationship_performance[str(name)] = export_performance_df(
                        dataframe=list_of_dfs_of_asset_prices[first_asset_df_index],
                        second_df=list_of_dfs_of_asset_prices[second_asset_df_index],
                        relationship=relationship,
                    )

    sorted_dict_return_only = sort_dict_desc(dict_of_relationship_performance, column_to_sort)
    sorted_dict = dict()

    for _ in sorted_dict_return_only.keys():
        sorted_dict[_] = dict_of_relationship_performance[_]

    # number_of_results is used to only export top X number of evaluations, it will export all combinations if set to 0
    if number_of_results is not 0:
        number_of_exports = number_of_results
    else:
        number_of_exports = len(sorted_dict)

    list_of_keys = list(sorted_dict.keys())
    # if export as csv is false the function will return the pairwise comparison along with relationship used in descending order : "1_6_xRelationship"
    if export_as_csv is True:
        for _ in sorted_dict.keys():
            if list_of_keys.index(_) == number_of_exports:
                break
            sorted_dict[_].to_csv(str(_) + "_performance_" + "#" + str(list_of_keys.index(_) + 1) + ".csv")
    else:
        return sorted_dict_return_only



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Copyright 2021 InferStat Ltd
# Created by: Thomas Oliver
# Created date:  19th October 2021

data_dictionary = {
    "fifty_fifty": {
        "function": "<function fifty_fifty at 0x102bb07b8>",
        "parameters": {},
        "series": [],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L37"
        },
    },
    "buy_and_hold": {
        "function": "<function buy_and_hold at 0x11944aa60>",
        "parameters": {},
        "series": [],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L43"
        },
    },
    "chande_kroll_crossover_strategy": {
        "function": "<function chande_kroll_crossover_strategy at 0x11944aae8>",
        "parameters": {},
        "series": ["high", "low", "price"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L49"
        },
    },
    "change_relationship": {
        "function": "<function change_relationship at 0x11944ac80>",
        "parameters": {},
        "series": ["price", "research"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L79"
        },
    },
    "change_relationship_oos": {
        "function": "change_relationship_OOS at 0x7f0a4fb86200>",
        "parameters": {},
        "series": ["price", "research"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L101"
        },
    },
    "combination_relationship": {
        "function": "<function combination_relationship at 0x11951ca60>",
        "parameters": {},
        "series": ["price", "research"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L156"
        },
    },
    "combination_relationship_oos": {
        "function": "<function combination_relationship_OOS at 0x7f0a4fd6dcb0>",
        "parameters": {},
        "series": ["price", "research"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L179"
        },
    },
    "difference_relationship": {
        "function": "<function difference_relationship at 0x11951cbf8>",
        "parameters": {},
        "series": ["price", "research"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L258"
        },
    },
    "difference_relationship_oos": {
        "function": "<function difference_relationship_OOS at 0x7f0a4fd6def0>",
        "parameters": {},
        "series": ["price", "research"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L279"
        },
    },
    "level_relationship": {
        "function": "<function level_relationship at 0x11951cd90>",
        "parameters": {},
        "series": ["price", "research"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L349"
        },
    },
    "level_relationship_oos": {
        "function": "<function level_relationship_OOS at 0x7f0a4fd7f170>",
        "parameters": {},
        "series": ["price", "research"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L370"
        },
    },
    "constant_allocation_size": {
        "function": "<function constant_allocation_size at 0x11951cb70>",
        "parameters": {"fixed_allocation_size": 1.0},
        "series": [],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L247"
        },
    },
    "high_low_difference": {
        "function": "<function high_low_difference at 0x11951cd08>",
        "parameters": {"scale": 1.0, "constant": 0.0},
        "series": ["high", "low"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L336"
        },
    },
    "sma_crossover_strategy": {
        "function": "<function sma_crossover_strategy at 0x11951cea0>",
        "parameters": {"fast": 0, "slow": 0},
        "series": ["price"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L423"
        },
    },
    "weighted_moving_averages": {
        "function": "<function weighted_moving_averages at 0x11951cf28>",
        "parameters": {
            "avg_price_coeff": 1.0,
            "avg_research_coeff": 1.0,
            "avg_price_length": 2,
            "avg_research_length": 2,
        },
        "series": ["price", "research"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L443"
        },
    },
    "change_regression": {
        "function": "<function change_regression at 0x119520048>",
        "parameters": {"change_coefficient": 0.1, "change_constant": 0.1},
        "series": ["research"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L486"
        },
    },
    "difference_regression": {
        "function": "<function difference_regression at 0x1195200d0>",
        "parameters": {"difference_coefficient": 0.1, "difference_constant": 0.1},
        "series": ["price", "research"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L502"
        },
    },
    "level_regression": {
        "function": "<function level_regression at 0x119520158>",
        "parameters": {"level_coefficient": 0.1, "level_constant": 0.1},
        "series": ["research"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L520"
        },
    },
    "level_and_change_regression": {
        "function": "<function level_and_change_regression at 0x1195201e0>",
        "parameters": {"level_coefficient": 0.1, "change_coefficient": 0.1, "level_and_change_constant": 0.1},
        "series": ["research"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L537"
        },
    },
    "buy_golden_cross_sell_death_cross": {
        "function": "<function buy_golden_cross_sell_death_cross at 0x119520268>",
        "parameters": {
            "allocation_size": 0.5,
            "deallocation_size": 0.5,
            "short_term_moving_avg_length": 50,
            "long_term_moving_avg_length": 200,
        },
        "series": ["price"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L563"
        },
    },
    "SMA_strategy": {
        "function": "<function SMA_strategy at 0x1195202f0>",
        "parameters": {"window": 1, "max_investment": 0.1},
        "series": ["close"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L596"
        },
    },
    "WMA_strategy": {
        "function": "<function WMA_strategy at 0x119520378>",
        "parameters": {"window": 1, "max_investment": 0.1},
        "series": ["close"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L610"
        },
    },
    "MACD_strategy": {
        "function": "<function MACD_strategy at 0x119520400>",
        "parameters": {"window_slow": 26, "window_fast": 12, "window_signal": 9, "max_investment": 0.1},
        "series": ["close"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L625"
        },
    },
    "RSI_strategy": {
        "function": "<function RSI_strategy at 0x119520488>",
        "parameters": {"window": 14, "max_investment": 0.1},
        "series": ["close"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L642"
        },
    },
    "stochastic_RSI_strategy": {
        "function": "<function stochastic_RSI_strategy at 0x119520510>",
        "parameters": {"window": 14, "max_investment": 0.1},
        "series": ["close"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L659"
        },
    },
    "EMA_strategy": {
        "function": "<function EMA_strategy at 0x119520598>",
        "parameters": {"window": 50, "max_investment": 0.1},
        "series": ["close"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L678"
        },
    },
    "bollinger_band_strategy": {
        "function": "<function bollinger_band_strategy at 0x119520620>",
        "parameters": {"window": 20, "window_dev": 2, "max_investment": 0.1},
        "series": ["close"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L692"
        },
    },
    "PPO_strategy": {
        "function": "<function PPO_strategy at 0x119520730>",
        "parameters": {"window_slow": 26, "window_fast": 12, "window_signal": 9, "max_investment": 0.1},
        "series": ["close"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L752"
        },
    },
    "PVO_strategy": {
        "function": "<function PVO_strategy at 0x1195207b8>",
        "parameters": {"window_slow": 26, "window_fast": 12, "window_signal": 9, "max_investment": 0.1},
        "series": ["volume"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L768"
        },
    },
    "TRIX_strategy": {
        "function": "<function TRIX_strategy at 0x119520840>",
        "parameters": {"window": 14, "max_investment": 0.1},
        "series": ["close"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L784"
        },
    },
    "TSI_strategy": {
        "function": "<function TSI_strategy at 0x1195208c8>",
        "parameters": {"window_slow": 25, "window_fast": 13, "window_signal": 13, "max_investment": 0.1},
        "series": ["close"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L798"
        },
    },
    "STC_strategy": {
        "function": "<function STC_strategy at 0x119520950>",
        "parameters": {
            "window_slow": 50,
            "window_fast": 23,
            "cycle": 10,
            "smooth1": 3,
            "smooth2": 3,
            "max_investment": 0.1,
        },
        "series": ["close"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L815"
        },
    },
    "KAMA_strategy": {
        "function": "<function KAMA_strategy at 0x1195209d8>",
        "parameters": {"window": 10, "pow1": 2, "pow2": 30, "max_investment": 0.1},
        "series": ["close"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L842"
        },
    },
    "aroon_strategy": {
        "function": "<function aroon_strategy at 0x119520a60>",
        "parameters": {"window": 25, "max_investment": 0.1},
        "series": ["close"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L861"
        },
    },
    "ROC_strategy": {
        "function": "<function ROC_strategy at 0x119520ae8>",
        "parameters": {"window": 12, "max_investment": 0.1},
        "series": ["close"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L882"
        },
    },
    "ADX_strategy": {
        "function": "<function ADX_strategy at 0x119520b70>",
        "parameters": {"window": 14, "max_investment": 0.1},
        "series": ["close", "high", "low"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L897"
        },
    },
    "vortex_strategy": {
        "function": "<function vortex_strategy at 0x119520bf8>",
        "parameters": {"window": 14, "max_investment": 0.1},
        "series": ["close", "high", "low"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L929"
        },
    },
    "DPO_strategy": {
        "function": "<function DPO_strategy at 0x1195206a8>",
        "parameters": {"window": 20, "max_investment": 0.1},
        "series": ["close"],
        "available_representation_types": {
            "github_permalink": "https://github.com/ta-oliver/infertrade/blob/6ce16a8518983587f25dc78118465973dfd92ad0/infertrade"
            "/algos/community/allocations.py#L738"
        },
    },
}
